/*
 * Host AP - driver interaction with ZyDAS 802.11 b/g driver
 * Copyright (c) 2004, Sam Leffler <sam@errno.com>
 * Copyright (c) 2004, Video54 Technologies
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Alternatively, this software may be distributed under the terms of BSD
 * license.
 *
 * See README and COPYING for more details.
 */
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <net/if_arp.h>
//#include <linux/wireless.h>
#include "wireless_copy.h"
#include <netinet/in.h>
#include <netpacket/packet.h>

#include "hostapd.h"
#include "driver.h"
#include "ieee802_1x.h"
#include "eloop.h"
#include "priv_netlink.h"
#include "sta_info.h"
#include "l2_packet.h"

#include "eapol_sm.h"
#include "wpa.h"
#include "radius.h"
#include "ieee802_11.h"
#include "accounting.h"
#include "zydas_common.h"
#include "hostap_common.h"
#include "common.h"
#ifdef SIMPLE_CONFIG
#include "wsc_ie.h"
#endif

struct zydas_driver_data {
	struct driver_ops ops;			/* base class */
	struct hostapd_data *hapd;		/* back pointer */

	char	iface[IFNAMSIZ + 1];
	int     ifindex;
	struct l2_packet_data *sock_xmit;	/* raw packet xmit socket */
	struct l2_packet_data *sock_recv;	/* raw packet recv socket */
	int	ioctl_sock;			/* socket for ioctl() use */
	int	wext_sock;			/* socket for wireless events */
    int     we_version;
};

static const struct driver_ops zydas_driver_ops;

static int zydas_ioctl(struct zydas_driver_data *drv, struct zydas_wlan_param *param,
			 int len, int show_err)
{
	struct iwreq iwr;

	memset(&iwr, 0, sizeof(iwr));
	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
	iwr.u.data.pointer = (caddr_t) param;
	iwr.u.data.length = len;

	if (ioctl(drv->ioctl_sock, ZD_IOCTL_WPA, &iwr) < 0) {
		int ret;
		ret = errno;
		if (show_err) 
			perror("ioctl[ZD_IOCTL_WPA]");
		return ret;
	}

	return 0;
}

static int zydas_get_wpa_ie(struct zydas_driver_data *drv, struct ieee80211req_wpaie *ie, int len)
{
	struct iwreq iwr;

	memset(&iwr, 0, sizeof(iwr));
	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
	iwr.u.data.pointer = (caddr_t) ie;
	iwr.u.data.length = len;

	if (ioctl(drv->ioctl_sock, ZD_IOCTL_GETWPAIE, &iwr) < 0) {
		int ret;
		ret = errno;
		perror("ioctl[ZD_IOCTL_GETWPAIE]");
		return ret;
	}

	return 0;
}

#ifdef SIMPLE_CONFIG
static int zydas_get_wsc_ie(struct zydas_driver_data *drv, struct ieee80211req_wscie *ie, int len)
{
	struct iwreq iwr;

	memset(&iwr, 0, sizeof(iwr));
	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
	iwr.u.data.pointer = (caddr_t) ie;
	iwr.u.data.length = len;

	if (ioctl(drv->ioctl_sock, ZD_IOCTL_GETWSCIE, &iwr) < 0) {
		int ret;
		ret = errno;
		perror("ioctl[ZD_IOCTL_GETWSCIE]");
		return ret;
	}

	return 0;
}

static int zydas_set_appiebuf(struct zydas_driver_data *drv, struct ieee80211req_getset_appiebuf *ie, int len)
{
	struct iwreq iwr;

	memset(&iwr, 0, sizeof(iwr));
	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
	iwr.u.data.pointer = (caddr_t) ie;
	iwr.u.data.length = len;

	if (ioctl(drv->ioctl_sock, ZD_IOCTL_SETAPPIEBUF, &iwr) < 0) {
		int ret;
		ret = errno;
		perror("ioctl[ZD_IOCTL_SETAPPIEBUF]");
		return ret;
	}

	return 0;
}
#endif

static int
zydas_set_param(struct zydas_driver_data *drv, int op, int arg, int show_err)
{
	struct iwreq iwr;
	int *i, ret = 0;

	memset(&iwr, 0, sizeof(iwr));
	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
	i = (int *) iwr.u.name;
	*i++ = op;
	*i++ = arg;

	if (ioctl(drv->ioctl_sock, ZD_IOCTL_PARAM, &iwr) < 0) {
		perror("ioctl[ZD_IOCTL_PARAM]");
		ret = -1;
	}

	return ret;
}

#ifdef SIMPLE_CONFIG
struct pbc_mac_t {
	u8 addr[ETH_ALEN];
	unsigned long timestamp;
};

struct pbc_count_t {
	u8 index; 
	struct pbc_mac_t mac[2];
} pbc_count;

extern int wsc_event_notify(char * pDataSend);
#endif

#if 0
static int
set80211priv(struct zydas_driver_data *drv, int op, void *data, int len)
{
#define	N(a)	(sizeof(a)/sizeof(a[0]))
	struct iwreq iwr;

	memset(&iwr, 0, sizeof(iwr));
	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
	if (len < IFNAMSIZ) {
		/*
		 * Argument data fits inline; put it there.
		 */
		memcpy(iwr.u.name, data, len);
	} else {
		/*
		 * Argument data too big for inline transfer; setup a
		 * parameter block instead; the kernel will transfer
		 * the data for the driver.
		 */
		iwr.u.data.pointer = data;
		iwr.u.data.length = len;
	}

	if (ioctl(drv->ioctl_sock, op, &iwr) < 0) {
		static const char *opnames[] = {
			"ioctl[IEEE80211_IOCTL_SETPARAM]",
			"ioctl[IEEE80211_IOCTL_GETPARAM]",
			"ioctl[IEEE80211_IOCTL_SETKEY]",
			"ioctl[SIOCIWFIRSTPRIV+3]",
			"ioctl[IEEE80211_IOCTL_DELKEY]",
			"ioctl[SIOCIWFIRSTPRIV+5]",
			"ioctl[IEEE80211_IOCTL_SETMLME]",
			"ioctl[SIOCIWFIRSTPRIV+7]",
			"ioctl[IEEE80211_IOCTL_SETOPTIE]",
			"ioctl[IEEE80211_IOCTL_GETOPTIE]",
			"ioctl[IEEE80211_IOCTL_ADDMAC]",
			"ioctl[SIOCIWFIRSTPRIV+11]",
			"ioctl[IEEE80211_IOCTL_DELMAC]",
			"ioctl[SIOCIWFIRSTPRIV+13]",
			"ioctl[IEEE80211_IOCTL_CHANLIST]",
			"ioctl[SIOCIWFIRSTPRIV+15]",
			"ioctl[IEEE80211_IOCTL_GETRSN]",
			"ioctl[SIOCIWFIRSTPRIV+17]",
			"ioctl[IEEE80211_IOCTL_GETKEY]",
		};
		op -= SIOCIWFIRSTPRIV;
		if (0 <= op && op < N(opnames))
			perror(opnames[op]);
		else
			perror("ioctl[unknown???]");
		return -1;
	}
	return 0;
#undef N
}
#endif
#if 0
static int
set80211param(struct zydas_driver_data *drv, int op, int arg)
{
	struct iwreq iwr;

	memset(&iwr, 0, sizeof(iwr));
	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
	iwr.u.mode = op;
	memcpy(iwr.u.name+sizeof(__u32), &arg, sizeof(arg));

	if (ioctl(drv->ioctl_sock, IEEE80211_IOCTL_SETPARAM, &iwr) < 0) {
		perror("ioctl[IEEE80211_IOCTL_SETPARAM]");
		return -1;
	}
	return 0;
}
#endif

static const char *
ether_sprintf(const u8 *addr)
{
	static char buf[sizeof(MACSTR)];

	if (addr != NULL)
		snprintf(buf, sizeof(buf), MACSTR, MAC2STR(addr));
	else
		snprintf(buf, sizeof(buf), MACSTR, 0,0,0,0,0,0);
	return buf;
}

/*
 * Configure WPA parameters.
 */
#if 0
static int
zydas_configure_wpa(struct zydas_driver_data *drv)
{
	hostapd *hapd = drv->hapd;
	struct hostapd_config *conf = hapd->conf;
	int v;

	switch (conf->wpa_group) {
	case WPA_CIPHER_CCMP:
		v = IEEE80211_CIPHER_AES_CCM;
		break;
	case WPA_CIPHER_TKIP:
		v = IEEE80211_CIPHER_TKIP;
		break;
	case WPA_CIPHER_WEP104:
		v = IEEE80211_CIPHER_WEP;
		break;
	case WPA_CIPHER_WEP40:
		v = IEEE80211_CIPHER_WEP;
		break;
	case WPA_CIPHER_NONE:
		v = IEEE80211_CIPHER_NONE;
		break;
	default:
		printf("Unknown group key cipher %u\n",
			conf->wpa_group);
		return -1;
	}
	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
		"%s: group key cipher=%d\n", __func__, v);
	if (set80211param(drv, IEEE80211_PARAM_MCASTCIPHER, v)) {
		printf("Unable to set group key cipher to %u\n", v);
		return -1;
	}
	if (v == IEEE80211_CIPHER_WEP) {
		/* key length is done only for specific ciphers */
		v = (conf->wpa_group == WPA_CIPHER_WEP104 ? 13 : 5);
		if (set80211param(drv, IEEE80211_PARAM_MCASTKEYLEN, v)) {
			printf("Unable to set group key length to %u\n", v);
			return -1;
		}
	}

	v = 0;
	if (conf->wpa_pairwise & WPA_CIPHER_CCMP)
		v |= 1<<IEEE80211_CIPHER_AES_CCM;
	if (conf->wpa_pairwise & WPA_CIPHER_TKIP)
		v |= 1<<IEEE80211_CIPHER_TKIP;
	if (conf->wpa_pairwise & WPA_CIPHER_NONE)
		v |= 1<<IEEE80211_CIPHER_NONE;
	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
		"%s: pairwise key ciphers=0x%x\n", __func__, v);
	if (set80211param(drv, IEEE80211_PARAM_UCASTCIPHERS, v)) {
		printf("Unable to set pairwise key ciphers to 0x%x\n", v);
		return -1;
	}

	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
		"%s: key management algorithms=0x%x\n",
		__func__, conf->wpa_key_mgmt);
	if (set80211param(drv, IEEE80211_PARAM_KEYMGTALGS, conf->wpa_key_mgmt)) {
		printf("Unable to set key management algorithms to 0x%x\n",
			conf->wpa_key_mgmt);
		return -1;
	}

	v = 0;
	if (conf->rsn_preauth)
		v |= BIT(0);
	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
		"%s: rsn capabilities=0x%x\n", __func__, conf->rsn_preauth);
	if (set80211param(drv, IEEE80211_PARAM_RSNCAPS, v)) {
		printf("Unable to set RSN capabilities to 0x%x\n", v);
		return -1;
	}

	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
		"%s: enable WPA= 0x%x\n", __func__, conf->wpa);
	if (set80211param(drv, IEEE80211_PARAM_WPA, conf->wpa)) {
		printf("Unable to set WPA to %u\n", conf->wpa);
		return -1;
	}
	return 0;
}
#endif

static int
zydas_set_iface_flags(void *priv, int dev_up)
{
	struct zydas_driver_data *drv = priv;
	hostapd *hapd = drv->hapd;
	struct ifreq ifr;

	HOSTAPD_DEBUG(HOSTAPD_DEBUG_VERBOSE,
		"%s: dev_up=%d\n", __func__, dev_up);

	if (drv->ioctl_sock < 0)
		return -1;

	memset(&ifr, 0, sizeof(ifr));
	snprintf(ifr.ifr_name, IFNAMSIZ, "%s", drv->iface);

	if (ioctl(drv->ioctl_sock, SIOCGIFFLAGS, &ifr) != 0) {
		perror("ioctl[SIOCGIFFLAGS]");
		return -1;
	}

	if (dev_up)
		ifr.ifr_flags |= IFF_UP;
	else
		ifr.ifr_flags &= ~IFF_UP;

	if (ioctl(drv->ioctl_sock, SIOCSIFFLAGS, &ifr) != 0) {
		perror("ioctl[SIOCSIFFLAGS]");
		return -1;
	}

	#if 0
	if (dev_up) {
		memset(&ifr, 0, sizeof(ifr));
		snprintf(ifr.ifr_name, IFNAMSIZ, "%s", drv->iface);
		ifr.ifr_mtu = HOSTAPD_MTU;
		if (ioctl(drv->ioctl_sock, SIOCSIFMTU, &ifr) != 0) {
			perror("ioctl[SIOCSIFMTU]");
			printf("Setting MTU failed - trying to survive with "
			       "current value\n");
		}
	}
	#endif

	return 0;
}

static int
zydas_set_ieee8021x(void *priv, int enabled)
{
	struct zydas_driver_data *drv = priv;
	hostapd *hapd = drv->hapd;
	struct hostapd_config *conf = hapd->conf;

	HOSTAPD_DEBUG(HOSTAPD_DEBUG_VERBOSE,
		"%s: enabled=%d\n", __func__, enabled);

#if 0
	if (!enabled) {
		/* XXX restore state */
		return set80211param(priv, IEEE80211_PARAM_AUTHMODE,
			IEEE80211_AUTH_AUTO);
	}
	if (!conf->wpa && !conf->ieee802_1x) {
		hostapd_logger(hapd, NULL, HOSTAPD_MODULE_DRIVER,
			HOSTAPD_LEVEL_WARNING, "No 802.1x or WPA enabled!");
		return -1;
	}
	if (conf->wpa && zydas_configure_wpa(drv) != 0) {
		hostapd_logger(hapd, NULL, HOSTAPD_MODULE_DRIVER,
			HOSTAPD_LEVEL_WARNING, "Error configuring WPA state!");
		return -1;
	}
	if (set80211param(priv, IEEE80211_PARAM_AUTHMODE,
		(conf->wpa ?  IEEE80211_AUTH_WPA : 
#ifdef SIMPLE_CONFIG
				((conf->wsc_auth_mode == HOSTAPD_AUTH_OPEN) ||
                                 (conf->wsc_auth_mode == WSC_AUTH_MODE_WEP)) ? 
                                IEEE80211_AUTH_OPEN : 
#endif
		IEEE80211_AUTH_8021X))) {
		hostapd_logger(hapd, NULL, HOSTAPD_MODULE_DRIVER,
			HOSTAPD_LEVEL_WARNING, "Error enabling WPA/802.1x!");
		return -1;
	}
#endif

#ifdef EAP_WSC
{
        char tmpCmdBuf[80]; 

        if(conf->wsc_auth_mode == WSC_AUTH_MODE_WEP)
        {
                sprintf(tmpCmdBuf, "iwconfig %s key \"[1]\" %s",
                                    conf->iface,conf->wsc_wep_key);
                printf("%s\n",tmpCmdBuf);
                system(tmpCmdBuf);  
                sprintf(tmpCmdBuf, "iwconfig %s key \"[1]\"",conf->iface);
                printf("%s\n",tmpCmdBuf);
                system(tmpCmdBuf);
        }
}
#endif

	return zydas_set_iface_flags(priv, 1);
}

static int
zydas_set_privacy(void *priv, int enabled)
{
	struct zydas_driver_data *drv = priv;
        wpa_printf (MSG_INFO, "zydas_set_privacy: %s", enabled ? "enabled": "disable");
	return zydas_set_param(drv, ZD_PARAM_PRIVACY, enabled, 1);
}

static int
zydas_set_sta_authorized(void *priv, u8 *addr, int authorized)
{
	struct zydas_driver_data *drv = priv;
	hostapd *hapd = drv->hapd;
	//struct ieee80211req_mlme mlme;

	HOSTAPD_DEBUG(HOSTAPD_DEBUG_VERBOSE,
		"%s: addr=%s authorized=%d\n",
		__func__, ether_sprintf(addr), authorized);

#if 0
	if (authorized)
		mlme.im_op = IEEE80211_MLME_AUTHORIZE;
	else
		mlme.im_op = IEEE80211_MLME_UNAUTHORIZE;
	mlme.im_reason = 0;
	memcpy(mlme.im_macaddr, addr, IEEE80211_ADDR_LEN);
	return set80211priv(priv, IEEE80211_IOCTL_SETMLME, &mlme,
			    sizeof(mlme));
#endif

	return 0;
}

#if 0
static int
zydas_del_key(void *priv, unsigned char *addr, int key_idx)
{
	struct zydas_driver_data *drv = priv;
	hostapd *hapd = drv->hapd;
	//struct ieee80211req_del_key wk;

	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
		"%s: addr=%s key_idx=%d\n",
		__func__, ether_sprintf(addr), key_idx);

	memset(&wk, 0, sizeof(wk));
	wk.idk_keyix = key_idx;
	if (addr != NULL)
		memcpy(wk.idk_macaddr, addr, IEEE80211_ADDR_LEN);

	return set80211priv(priv, IEEE80211_IOCTL_DELKEY, &wk, sizeof(wk));
}
#endif

static int
zydas_set_key(void *priv, const char *alg,
	     unsigned char *addr, int key_idx,
	     u8 *key, size_t key_len)
{
	struct zydas_driver_data *drv = priv;
	struct zydas_wlan_param *param;
	u8 *buf;
	size_t blen;
	int ret = 0;
	char alg_name[ZD_CRYPT_ALG_NAME_LEN];

        memset(alg_name,0,ZD_CRYPT_ALG_NAME_LEN);
	if(strcmp(alg, "none") == 0)
		strcpy(alg_name,"NONE");
	else if(strcmp(alg, "WEP") == 0)
		strcpy(alg_name,"WEP");
	else if(strcmp(alg, "TKIP") == 0)
		strcpy(alg_name,"TKIP");
	else if(strcmp(alg, "CCMP") == 0)
		strcpy(alg_name,"CCMP");
	else {
		wpa_printf(MSG_DEBUG, "Unknown encryption algorithm: %s\n", alg);
		return -1;
	}

	wpa_printf(MSG_INFO, "%s: alg=%s key_idx=%d "
		   "key_len=%d", __FUNCTION__, alg_name, key_idx, key_len);

	//if (seq_len > 8)
	//	return -2;

	blen = sizeof(*param) + key_len;
	buf = malloc(blen);
	if (buf == NULL)
		return -1;
	memset(buf, 0, blen);

	param = (struct zydas_wlan_param *) buf;
	param->cmd = ZD_CMD_SET_ENCRYPT_KEY;

#if 1
	if (addr == NULL) {
		memset(param->sta_addr, 0xff, ETH_ALEN);
		wpa_printf(MSG_INFO, "zydas_set_key:Group address: \n");
	}
	else {
		memcpy(param->sta_addr, addr, ETH_ALEN);
		wpa_printf(MSG_INFO, "zydas_set_key:Unicast address: 0x%02x:0x%02x:0x%02x:0x%02x:0x%02x:0x%02x\n", addr[0], addr[1], addr[2], addr[3], addr[4], addr[5]);
	}
#else
	memset(param->sta_addr, 0xff, ETH_ALEN);
#endif
	strncpy((char*)param->u.crypt.alg, (char*)alg_name, ZD_CRYPT_ALG_NAME_LEN);
	//param->u.crypt.flags = set_tx ? ZD_FLAG_SET_TX_KEY : 0;
	param->u.crypt.flags = ZD_FLAG_SET_TX_KEY;
	param->u.crypt.idx = key_idx;
	//memcpy(param->u.crypt.seq, seq, seq_len);
	param->u.crypt.key_len = key_len;
	memcpy((u8 *) param->u.crypt.key, key, key_len);

	/* Dump key context */
	if (key_len > 0)
	{
		int ii;

		wpa_printf(MSG_DEBUG, "Key Context:");
		for(ii = 0; ii < key_len; ) {
			printf("0x%02x ", key[ii]);

			if((++ii % 16) == 0)
				printf("\n");
		}

		printf("\n");
	}

	if (zydas_ioctl(drv, param, blen, 1)) {
		wpa_printf(MSG_WARNING, "Failed to set encryption.");
		ret = -1;
	}
	free(buf);

	return ret;
}


static int
zydas_get_seqnum(void *priv, u8 *addr, int idx, u8 *seq)
{
	struct zydas_driver_data *drv = priv;
	struct zydas_wlan_param *param;
	int blen;
	int ret = 0;

	blen = sizeof(*param);
	param = (struct zydas_wlan_param *) malloc(blen);
	if (param == NULL)
		return -1;

	memset(param, 0, blen);
	param->cmd = ZD_CMD_GET_TSC;

	if(addr == NULL)
		memset(param->sta_addr, 0xff, ETH_ALEN);
	else
		memcpy(param->sta_addr, addr, ETH_ALEN);

	ret = zydas_ioctl(drv, param, blen, 1);
	if(ret == 0)
		memcpy(seq, param->u.crypt.seq, 8);
	else
		printf("Failed to get TSC from driver\n");

	free(param);
	return ret;
}


static int 
zydas_flush(void *priv)
{
#if 0
	struct zydas_driver_data *drv = priv;
	struct prism2_hostapd_param param;

	memset(&param, 0, sizeof(param));
	param.cmd = PRISM2_HOSTAPD_FLUSH;
	return zydas_ioctl(drv, &param, sizeof(param));
#else
	return 0;		/* XXX */
#endif
}


static int
zydas_read_sta_driver_data(void *priv, struct hostap_sta_driver_data *data,
					u8 *addr)
{
	//struct zydas_driver_data *drv = priv;
	//char buf[1024], line[128], *pos;
	//FILE *f;
	//unsigned long val;

	memset(data, 0, sizeof(*data));
#if 0
	snprintf(buf, sizeof(buf), "/proc/net/hostap/%s/" MACSTR,
		 drv->iface, MAC2STR(addr));

	f = fopen(buf, "r");
	if (!f)
		return -1;
	/* Need to read proc file with in one piece, so use large enough
	 * buffer. */
	setbuffer(f, buf, sizeof(buf));

	while (fgets(line, sizeof(line), f)) {
		pos = strchr(line, '=');
		if (!pos)
			continue;
		*pos++ = '\0';
		val = strtoul(pos, NULL, 10);
		if (strcmp(line, "rx_packets") == 0)
			data->rx_packets = val;
		else if (strcmp(line, "tx_packets") == 0)
			data->tx_packets = val;
		else if (strcmp(line, "rx_bytes") == 0)
			data->rx_bytes = val;
		else if (strcmp(line, "tx_bytes") == 0)
			data->tx_bytes = val;
	}

	fclose(f);
#endif

	return 0;
}

static int
zydas_set_opt_ie(void *priv, const u8 *wpa_ie, size_t wpa_ie_len)
{
	struct zydas_driver_data *drv = priv;
	struct zydas_wlan_param *param;
	int res;
	size_t blen = ZD_GENERIC_ELEMENT_HDR_LEN + wpa_ie_len;
	if (blen < sizeof(*param))
		blen = sizeof(*param);

	param = (struct zydas_wlan_param *) malloc(blen);
	if (param == NULL)
		return -1;

	memset(param, 0, blen);
	param->cmd = ZD_CMD_SET_GENERIC_ELEMENT;
	param->u.generic_elem.len = wpa_ie_len;
	memcpy(param->u.generic_elem.data, wpa_ie, wpa_ie_len);
	res = zydas_ioctl(drv, param, blen, 1);
	free(param);

	return res;
}

static int
zydas_sta_deauth(void *priv, u8 *addr, int reason_code)
{
	struct zydas_driver_data *drv = priv;
	struct zydas_wlan_param *param;
	int blen;
	int ret;

	blen = sizeof(*param);
	param = (struct zydas_wlan_param *) malloc(blen);
	if (param == NULL)
		return -1;

	memset(param, 0, blen);
	param->cmd = ZD_CMD_SET_MLME;
	param->u.mlme.cmd = MLME_STA_DEAUTH;
	param->u.mlme.reason_code = reason_code;
        
	memcpy(param->sta_addr, addr, ETH_ALEN);
	ret = zydas_ioctl(drv, param, blen, 1);
        if (reason_code == WLAN_REASON_MICHAEL_MIC_FAILURE)
        {  // start counterMeasure period
            //zydas_set_param(drv, ZD_PARAM_COUNTERMEASURES, 1, 1);
        }
        usleep(100000);
	free(param);

	return ret;
}

static int
zydas_sta_disassoc(void *priv, u8 *addr, int reason_code)
{
	struct zydas_driver_data *drv = priv;
	struct zydas_wlan_param *param;
	int blen;
	int ret;

	blen = sizeof(*param);
	param = (struct zydas_wlan_param *) malloc(blen);
	if (param == NULL)
		return -1;

	memset(param, 0, blen);
	param->cmd = ZD_CMD_SET_MLME;
	param->u.mlme.cmd = MLME_STA_DISASSOC;
	param->u.mlme.reason_code = reason_code;
	memcpy(param->sta_addr, addr, ETH_ALEN);
	ret = zydas_ioctl(drv, param, blen, 1);
	free(param);

	return ret;
}

static int
zydas_del_sta(struct zydas_driver_data *drv, u8 addr[IEEE80211_ADDR_LEN])
{
	struct hostapd_data *hapd = drv->hapd;
	struct sta_info *sta;

	hostapd_logger(hapd, addr, HOSTAPD_MODULE_IEEE80211,
		HOSTAPD_LEVEL_INFO, "deassociated");

	sta = ap_get_sta(hapd, addr);
	if (sta != NULL) {
		sta->flags &= ~WLAN_STA_ASSOC;
		wpa_sm_event(hapd, sta, WPA_DISASSOC);
		sta->acct_terminate_cause = RADIUS_ACCT_TERMINATE_CAUSE_USER_REQUEST;
		ieee802_1x_set_port_enabled(hapd, sta, 0);
		ap_free_sta(hapd, sta);
	}
	return 0;
}

#ifdef SIMPLE_CONFIG
static int
zydas_set_wsc_ie(void *priv, u8 *iebuf, int iebuflen, u32 frametype)
{

	struct zydas_driver_data *drv = priv;
	hostapd *hapd = drv->hapd;
	u8 buf[IEEE80211_APPIE_MAX];
	struct ieee80211req_getset_appiebuf * beac_ie;

	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL, "%s buflen = %d\n", 
			__func__, iebuflen);

	beac_ie = (struct ieee80211req_getset_appiebuf *) buf;
	beac_ie->app_frmtype = frametype;
	beac_ie->app_buflen = iebuflen;
	memcpy(&(beac_ie->app_buf[0]), iebuf, iebuflen);

	return zydas_set_appiebuf(drv,beac_ie,sizeof(struct ieee80211req_getset_appiebuf) + iebuflen);
}

static int
zydas_set_wsc_beacon_ie(void *priv, u8 *iebuf, int iebuflen)
{
	return zydas_set_wsc_ie(priv, iebuf, iebuflen, 
			IEEE80211_APPIE_FRAME_BEACON);
}

static int
zydas_set_wsc_probe_resp_ie(void *priv, u8 *iebuf, int iebuflen)
{
	return zydas_set_wsc_ie(priv, iebuf, iebuflen, 
			IEEE80211_APPIE_FRAME_PROBE_RESP);
}

#endif

static int
zydas_process_wpa_ie(struct zydas_driver_data *drv, struct sta_info *sta)
{
	struct hostapd_data *hapd = drv->hapd;
	struct ieee80211req_wpaie ie;
	int ielen, res;

	/*
	 * Fetch negotiated WPA/RSN parameters from the system.
	 */
	memset(&ie, 0, sizeof(ie));
	memcpy(ie.wpa_macaddr, sta->addr, IEEE80211_ADDR_LEN);
	if (zydas_get_wpa_ie(drv, &ie, sizeof(ie))) {
		printf("Failed to get WPA/RSN information element.\n");
		return -1;		/* XXX not right */
	}
	ielen = ie.wpa_ie[1];
	if (ielen == 0) {
		printf("No WPA/RSN information element for station!?\n");
#ifdef WSC_AKM_NO_1X
		return WPA_NO_IE;		/* XXX not right */
#else
                return -1;
#endif
	}
	ielen += 2;
	res = wpa_validate_wpa_ie(hapd, sta, ie.wpa_ie, ielen,
			ie.wpa_ie[0] == WLAN_EID_RSN ?
			    HOSTAPD_WPA_VERSION_WPA2 : HOSTAPD_WPA_VERSION_WPA);
	if (res != WPA_IE_OK) {
		printf("WPA/RSN information element rejected? (res %u)\n", res);
		return -1;
	}
	free(sta->wpa_ie);
	sta->wpa_ie = malloc(ielen);
	if (sta->wpa_ie == NULL) {
		printf("No memory to save WPA/RSN information element!\n");
		return -1;
	}
	memcpy(sta->wpa_ie, ie.wpa_ie, ielen);
	sta->wpa_ie_len = ielen;
	return 0;
}

#ifdef WSC_AKM_NO_1X
static int
zydas_process_wsc_ie(struct zydas_driver_data *drv, struct sta_info *sta)
{
	struct ieee80211req_wscie ie;
	int ielen;

	/*
	 * Fetch WSC association info from the system.
	 */
	memset(&ie, 0, sizeof(ie));
	memcpy(ie.wsc_macaddr, sta->addr, IEEE80211_ADDR_LEN);
	if (zydas_get_wsc_ie(drv, &ie, sizeof(ie))) {
		printf("Failed to get WSC information element.\n");
		return -1;		/* XXX not right */
	}
	ielen = ie.wsc_ie[1];
	if (ielen != 0) {
		printf("WSC information element for station!?\n");
		return ASSO_HAS_WSC_IE;		/* XXX not right */
	}  
	return 0;
}
#endif

static int
zydas_new_sta(struct zydas_driver_data *drv, u8 addr[IEEE80211_ADDR_LEN])
{
	struct hostapd_data *hapd = drv->hapd;
	struct sta_info *sta;
	int new_assoc;
#ifdef WSC_AKM_NO_1X
        int ret;
#endif

        printf("------------zydas_new_sta\n");
	hostapd_logger(hapd, addr, HOSTAPD_MODULE_IEEE80211,
		HOSTAPD_LEVEL_INFO, "associated");

	sta = ap_get_sta(hapd, addr);
	if (sta) {
		accounting_sta_stop(hapd, sta);
	} else {
		sta = ap_sta_add(hapd, addr);
		if (sta == NULL)
			return -1;
	}
	accounting_sta_get_id(hapd, sta);

	if (hapd->conf->wpa) {
#ifdef WSC_AKM_NO_1X
		ret = zydas_process_wpa_ie(drv, sta);
		if (ret) {
                        if (ret==WPA_NO_IE)
                        {
                            sta->wpa_ie = NULL;
                            sta->wpa_ie_len = 0;
                            sta->flags |= WLAN_STA_WSC_ASSOC;
                            printf("set WLAN_STA_WSC_ASSOC flag\n");
                        }  
                        else
                            return -1;
                }
                if(zydas_process_wsc_ie(drv, sta)==ASSO_HAS_WSC_IE)
                {
                       sta->flags |= WLAN_STA_WSC_ASSOC;
                       printf("set WLAN_STA_WSC_ASSOC WSC IE flag\n");
 
                }
#else
		if (zydas_process_wpa_ie(drv, sta))
			return -1;
#endif
	} else {
		free(sta->wpa_ie);
		sta->wpa_ie = NULL;
		sta->wpa_ie_len = 0;
	}

	/*
	 * Now that the internal station state is setup
	 * kick the authenticator into action.
	 */
	new_assoc = (sta->flags & WLAN_STA_ASSOC) == 0;
	sta->flags |= WLAN_STA_ASSOC;
	wpa_sm_event(hapd, sta, WPA_ASSOC);
	hostapd_new_assoc_sta(hapd, sta, !new_assoc);
		
	ieee802_1x_notify_port_enabled(sta->eapol_sm, 1);
	return 0;
}

#ifdef SIMPLE_CONFIG
static int pbc_check_overlap()
{
    unsigned long currTime;
    currTime = time(NULL);
  
    if( (time(NULL)-pbc_count.mac[0].timestamp<120) && 
        (time(NULL)-pbc_count.mac[1].timestamp<120) ){
        printf(" ******* overlap\n");
        return -1;
    } else {
        return 0;
    }
}

static void
zydas_push_button_notify()
{

    /* Send a push_button event to WSC ,including PBC overlap detection*/
    if(pbc_check_overlap()==0) {
        wsc_event_notify("PUSH_BUTTON_NOOVERLAP");
    }else {
        wsc_event_notify("PUSH_BUTTON_OVERLAP");
    }
      
}

#endif

static void
zydas_wireless_event_wireless_custom(struct zydas_driver_data *drv,
				       char *custom)
{
	struct hostapd_data *hapd = drv->hapd;

	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL, "Custom wireless event: '%s'\n",
		      custom);

	if (strncmp(custom, "MLME-MICHAELMICFAILURE.indication", 33) == 0) {
		char *pos;
		u8 addr[ETH_ALEN];
		pos = strstr(custom, "addr=");
		if (pos == NULL) {
			HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
				      "MLME-MICHAELMICFAILURE.indication "
				      "without sender address ignored\n");
			return;
		}
		pos += 5;
		if (hwaddr_aton(pos, addr) == 0) {
			ieee80211_michael_mic_failure(drv->hapd, addr, 1);
		} else {
			HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
				      "MLME-MICHAELMICFAILURE.indication "
				      "with invalid MAC address");
		}
	}
#ifdef SIMPLE_CONFIG
	else if (strncmp(custom, "PUSH-BUTTON.indication", 22) == 0) {
		zydas_push_button_notify();
    	}
#endif
}

static void
zydas_wireless_event_wireless(struct zydas_driver_data *drv,
					    char *data, int len)
{
#if 0
	struct hostapd_data *hapd = drv->hapd;
	struct iw_event *iwe;
	char *pos, *end, *custom, *buf;

	pos = data;
	end = data + len;

	while (pos + IW_EV_LCP_LEN <= end) {
		iwe = (struct iw_event *) pos;
		printf("Wireless event: "
			      "cmd=0x%x len=%d\n", iwe->cmd, iwe->len);
		if (iwe->len <= IW_EV_LCP_LEN)
        {
            printf("iwe->len <= IW_EV_LCP_LEN=%d\n", IW_EV_LCP_LEN);
			return;
        }
		switch (iwe->cmd) {
		case IWEVEXPIRED:
			zydas_del_sta(drv, (u8*)iwe->u.addr.sa_data);
			break;
		case IWEVREGISTERED:
			zydas_new_sta(drv, (u8*)iwe->u.addr.sa_data);
			break;
		case IWEVCUSTOM:
			custom = pos + IW_EV_POINT_LEN;
			if (custom + iwe->u.data.length > end)
				return;
			buf = malloc(iwe->u.data.length + 1);
			if (buf == NULL)
				return;		/* XXX */
			memcpy(buf, custom, iwe->u.data.length);
			buf[iwe->u.data.length] = '\0';
			zydas_wireless_event_wireless_custom(drv, buf);
			free(buf);
			break;
		}

		pos += iwe->len;
	}
#else
    struct hostapd_data *hapd = drv->hapd;
    struct iw_event iwe_buf, *iwe = &iwe_buf;
    char *pos, *end, *custom, *buf;

    pos = data;
    end = data + len;

    while (pos + IW_EV_LCP_LEN <= end) {
        memcpy(&iwe_buf, pos, IW_EV_LCP_LEN);
        HOSTAPD_DEBUG(HOSTAPD_DEBUG_VERBOSE, "Wireless event: "
                  "cmd=0x%x len=%d\n", iwe->cmd, iwe->len);
        if (iwe->len <= IW_EV_LCP_LEN)
            return;

        custom = pos + IW_EV_POINT_LEN;
        if (drv->we_version > 18  &&
            (iwe->cmd == IWEVMICHAELMICFAILURE ||
             iwe->cmd == IWEVCUSTOM)) {
            /* WE-19 removed the pointer from struct iw_point */
            char *dpos = (char *) &iwe_buf.u.data.length;
            int dlen = dpos - (char *) &iwe_buf;
            memcpy(dpos, pos + IW_EV_LCP_LEN,
                   sizeof(struct iw_event) - dlen);
        } else {
            memcpy(&iwe_buf, pos, sizeof(struct iw_event));
            custom += IW_EV_POINT_OFF;
        }

        switch (iwe->cmd) {
        case IWEVEXPIRED:
            zydas_del_sta(drv, (u8 *) iwe->u.addr.sa_data);
            break;
        case IWEVREGISTERED:
            zydas_new_sta(drv, (u8 *) iwe->u.addr.sa_data);
            break;
        case IWEVCUSTOM:
            if (custom + iwe->u.data.length > end)
                return;
            buf = malloc(iwe->u.data.length + 1);
            if (buf == NULL)
                return;     /* XXX */
            memcpy(buf, custom, iwe->u.data.length);
            buf[iwe->u.data.length] = '\0';
            zydas_wireless_event_wireless_custom(drv, buf);
            free(buf);
            break;
        }
        pos += iwe->len;
    }
#endif

}


static void
zydas_wireless_event_rtm_newlink(struct zydas_driver_data *drv,
					       struct nlmsghdr *h, int len)
{
	struct ifinfomsg *ifi;
	int attrlen, nlmsg_len, rta_len;
	struct rtattr * attr;

	if (len < sizeof(*ifi))
		return;

	ifi = NLMSG_DATA(h);

	if (ifi->ifi_index != drv->ifindex)
		return;

	nlmsg_len = NLMSG_ALIGN(sizeof(struct ifinfomsg));

	attrlen = h->nlmsg_len - nlmsg_len;
	if (attrlen < 0)
		return;

	attr = (struct rtattr *) (((char *) ifi) + nlmsg_len);

	rta_len = RTA_ALIGN(sizeof(struct rtattr));
	while (RTA_OK(attr, attrlen)) {
		if (attr->rta_type == IFLA_WIRELESS) {
			zydas_wireless_event_wireless(
				drv, ((char *) attr) + rta_len,
				attr->rta_len - rta_len);
		}
		attr = RTA_NEXT(attr, attrlen);
	}
}


static void
zydas_wireless_event_receive(int sock, void *eloop_ctx, void *sock_ctx)
{
	char buf[256];
	int left;
	struct sockaddr_nl from;
	socklen_t fromlen;
	struct nlmsghdr *h;
	struct zydas_driver_data *drv = eloop_ctx;

	fromlen = sizeof(from);
	left = recvfrom(sock, buf, sizeof(buf), MSG_DONTWAIT,
			(struct sockaddr *) &from, &fromlen);
	if (left < 0) {
		if (errno != EINTR && errno != EAGAIN)
			perror("recvfrom(netlink)");
		return;
	}

	h = (struct nlmsghdr *) buf;
	while (left >= sizeof(*h)) {
		int len, plen;

		len = h->nlmsg_len;
		plen = len - sizeof(*h);
		if (len > left || plen < 0) {
			printf("Malformed netlink message: "
			       "len=%d left=%d plen=%d\n",
			       len, left, plen);
			break;
		}

		switch (h->nlmsg_type) {
		case RTM_NEWLINK:
			zydas_wireless_event_rtm_newlink(drv, h, plen);
			break;
		}

		len = NLMSG_ALIGN(len);
		left -= len;
		h = (struct nlmsghdr *) ((char *) h + len);
	}

	if (left > 0) {
		printf("%d extra bytes in the end of netlink message\n", left);
	}
}

zydas_get_we_version(struct zydas_driver_data *drv)
{
    struct iw_range *range;
    struct iwreq iwr;
    int minlen;
    size_t buflen;

    drv->we_version = 0;

    /*
     * Use larger buffer than struct iw_range in order to allow the
     * structure to grow in the future.
     */
    buflen = sizeof(struct iw_range) + 500;
    range = malloc(buflen);
    if (range == NULL)
        return -1;
    memset(range, 0, buflen);

    memset(&iwr, 0, sizeof(iwr));
    strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
    iwr.u.data.pointer = (caddr_t) range;
    iwr.u.data.length = buflen;

    minlen = ((char *) &range->enc_capa) - (char *) range +
        sizeof(range->enc_capa);

    if (ioctl(drv->ioctl_sock, SIOCGIWRANGE, &iwr) < 0) {
        perror("ioctl[SIOCGIWRANGE]");
        free(range);
        return -1;
    } else if (iwr.u.data.length >= minlen &&
           range->we_version_compiled >= 18) {
        wpa_printf(MSG_DEBUG, "SIOCGIWRANGE: WE(compiled)=%d "
               "WE(source)=%d enc_capa=0x%x",
               range->we_version_compiled,
               range->we_version_source,
               range->enc_capa);
        drv->we_version = range->we_version_compiled;
    }

    free(range);
    return 0;
}
static int
zydas_wireless_event_init(void *priv)
{
	struct zydas_driver_data *drv = priv;
	int s;
	struct sockaddr_nl local;

    zydas_get_we_version(drv);
	drv->wext_sock = -1;

	s = socket(PF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
	if (s < 0) {
		perror("socket(PF_NETLINK,SOCK_RAW,NETLINK_ROUTE)");
		return -1;
	}

	memset(&local, 0, sizeof(local));
	local.nl_family = AF_NETLINK;
	local.nl_groups = RTMGRP_LINK;
	if (bind(s, (struct sockaddr *) &local, sizeof(local)) < 0) {
		perror("bind(netlink)");
		close(s);
		return -1;
	}

	eloop_register_read_sock(s, zydas_wireless_event_receive, drv, NULL);
	drv->wext_sock = s;

	return 0;
}


static void
zydas_wireless_event_deinit(void *priv)
{
	struct zydas_driver_data *drv = priv;

	if (drv != NULL) {
		if (drv->wext_sock < 0)
			return;
		eloop_unregister_read_sock(drv->wext_sock);
		close(drv->wext_sock);
	}
}


static int
zydas_send_eapol(void *priv, u8 *addr, u8 *data, size_t data_len, int encrypt)
{
	struct zydas_driver_data *drv = priv;
	hostapd *hapd = drv->hapd;
	unsigned char buf[3000];
	unsigned char *bp = buf;
	struct l2_ethhdr *eth;
	size_t len;
	int status;

	/*
	 * Prepend the Etherent header.  If the caller left us
	 * space at the front we could just insert it but since
	 * we don't know we copy to a local buffer.  Given the frequency
	 * and size of frames this probably doesn't matter.
	 */
	len = data_len + sizeof(struct l2_ethhdr);
	if (len > sizeof(buf)) {
		bp = malloc(len);
		if (bp == NULL) {
			printf("EAPOL frame discarded, cannot malloc temp "
				"buffer of size %u!\n", len);
			return -1;
		}
	}
	eth = (struct l2_ethhdr *) bp;
	memcpy(eth->h_dest, addr, ETH_ALEN);
	memcpy(eth->h_source, drv->hapd->own_addr, ETH_ALEN);
	eth->h_proto = htons(ETH_P_EAPOL);
	memcpy(eth+1, data, data_len);

	if (HOSTAPD_DEBUG_COND(HOSTAPD_DEBUG_MSGDUMPS))
		hostapd_hexdump("TX EAPOL", bp, len);

	//status = l2_packet_send(drv->sock_xmit, addr, bp, len);
        status = l2_packet_send(drv->sock_xmit, addr, ETH_P_EAPOL, bp, len);

	if (bp != buf)
		free(bp);
	return status;
}

static void
handle_read(void *ctx, const u8 *src_addr, const u8 *buf, size_t len)
{
        struct zydas_driver_data *drv = ctx;
        hostapd *hapd = drv->hapd;
        struct sta_info *sta;

        sta = ap_get_sta(hapd, src_addr);
        if (!sta || !(sta->flags & WLAN_STA_ASSOC)) {
                printf("Data frame from not associated STA %s\n",
                       ether_sprintf(src_addr));
                /* XXX cannot happen */
                return;
        }
        ieee802_1x_receive(hapd, src_addr, buf + sizeof(struct l2_ethhdr),
                           len - sizeof(struct l2_ethhdr));
}

#ifdef SIMPLE_CONFIG
static int
wpa_driver_zydas_process_frame(void *ctx, const unsigned char *src_addr, const unsigned char *buf, size_t len, unsigned char *newbuf, int *newlen)
{
	u8 * frm;
	u8 * endfrm;
	char ssid[32];
	u8 bssid[6];
	struct ieee80211_hdr *wh;
	
    	wh = (struct ieee80211_hdr *) buf;
	frm = (u8 *)&wh[1];
	endfrm = (u8 *) (buf + len);
	memcpy(bssid, wh->addr3, IEEE80211_ADDR_LEN);

	// get ssid
	while (frm < endfrm)
	{
		switch (*frm)
		{
		case IEEE80211_ELEMID_SSID:
			if (frm[1] > 1) {
				memset(ssid, 0, 32);
				memcpy(ssid, &frm[2], frm[1]);
			} 
			break;
		case IEEE80211_ELEMID_VENDOR:
		    	// check for WFA OUI
			if (frm[2] == 0x00 && frm[3] == 0x50 && frm[4] == 0xF2 && frm[5]==0x04)
			{
				printf("Got WSC IE; len = %d\n", frm[1]);
                 		if(frm[8]!=0x00 || frm[9]!=0x80) {
                    			printf(" not PBC probe request\n");
                    			break;
            			}
            			if( (memcmp(wh->addr2,pbc_count.mac[0].addr,ETH_ALEN) !=0) &&
                     			(memcmp(wh->addr2,pbc_count.mac[2].addr,ETH_ALEN) !=0)) 
            			{
					 // new PBC station, save it
					 if ((pbc_count.index %2) == 0) {
					     memcpy(pbc_count.mac[0].addr, wh->addr2, ETH_ALEN);
					     pbc_count.mac[0].timestamp = time(NULL);
					 } else {
					     memcpy(pbc_count.mac[1].addr, wh->addr2, ETH_ALEN);
					     pbc_count.mac[1].timestamp = time(NULL);
					 }
					 pbc_count.index ++;
            			}	 
            			break;
			}
		default:
			break;
		} // switch
		frm += frm[1] + 2;
	} // while

	return 0;
}

static void
wsc_handle_frames(void *ctx, const unsigned char *src_addr, const unsigned char *buf, size_t len)
{
	u8 newbuf[256];
	int newlen = 0;

	if (wpa_driver_zydas_process_frame(ctx, src_addr, 
				buf, len, newbuf + sizeof(WSC_IE_COMMAND_DATA), &newlen) < 0)
	{
        	wpa_printf(MSG_ERROR, "process_frame was not called or successful\n");
		return;
	}

}
#endif

static int
zydas_init(struct hostapd_data *hapd)
{
	struct zydas_driver_data *drv;
	struct ifreq ifr;
	struct iwreq iwr;

	drv = malloc(sizeof(struct zydas_driver_data));
	if (drv == NULL) {
		printf("Could not allocate memory for zydas driver data\n");
		goto bad;
	}

	memset(drv, 0, sizeof(*drv));
	drv->ops = zydas_driver_ops;
	drv->hapd = hapd;
	drv->ioctl_sock = socket(PF_INET, SOCK_DGRAM, 0);
	if (drv->ioctl_sock < 0) {
		perror("socket[PF_INET,SOCK_DGRAM]");
		goto bad;
	}
	memcpy(drv->iface, hapd->conf->iface, sizeof(drv->iface));

	memset(&ifr, 0, sizeof(ifr));
	snprintf(ifr.ifr_name, sizeof(ifr.ifr_name), "%s", drv->iface);
	if (ioctl(drv->ioctl_sock, SIOCGIFINDEX, &ifr) != 0) {
		perror("ioctl(SIOCGIFINDEX)");
		goto bad;
	}
	drv->ifindex = ifr.ifr_ifindex;

	drv->sock_xmit = l2_packet_init(drv->iface, NULL, ETH_P_EAPOL,
				handle_read, drv, 1);
	if (drv->sock_xmit == NULL)
		goto bad;
	if (l2_packet_get_own_addr(drv->sock_xmit, hapd->own_addr))
		goto bad;
	if (hapd->conf->bridge[0] != '\0') {
		HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
			"Configure bridge %s for EAPOL traffic.\n",
			hapd->conf->bridge);
		drv->sock_recv = l2_packet_init(hapd->conf->bridge, NULL,
						ETH_P_EAPOL, handle_read, drv,1);
		if (drv->sock_recv == NULL)
			goto bad;
	} else
		drv->sock_recv = drv->sock_xmit;

	memset(&iwr, 0, sizeof(iwr));
	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);

	iwr.u.mode = IW_MODE_MASTER;

	zydas_set_iface_flags(drv, 1);/*mark down during setup */
	if (ioctl(drv->ioctl_sock, SIOCSIWMODE, &iwr) < 0) {
		perror("ioctl[SIOCSIWMODE]");
		printf("Could not set interface to master mode!\n");
		goto bad;
	}
	hapd->driver = &drv->ops;

#ifdef SIMPLE_CONFIG
    	l2_packet_init(drv->iface, NULL, 0x0019/*ETH_P_EAPOL*/,
				wsc_handle_frames, drv, 1); 
#endif

	return 0;
bad:
	if (drv->sock_xmit != NULL)
		l2_packet_deinit(drv->sock_xmit);
	if (drv->ioctl_sock >= 0)
		close(drv->ioctl_sock);
	if (drv != NULL)
		free(drv);
	return -1;
}

static void
zydas_deinit(void *priv)
{
	struct zydas_driver_data *drv = priv;

	drv->hapd->driver = NULL;
wpa_printf(MSG_DEBUG, "zydas_deinit...\n");
        zydas_set_param(drv, ZD_PARAM_COUNTERMEASURES, 0, 1); //Stop counterMeasures state.
	(void) zydas_set_iface_flags(drv, 0);
	if (drv->ioctl_sock >= 0)
		close(drv->ioctl_sock);
	if (drv->sock_recv != NULL && drv->sock_recv != drv->sock_xmit)
		l2_packet_deinit(drv->sock_recv);
	if (drv->sock_xmit != NULL)
		l2_packet_deinit(drv->sock_xmit);
	free(drv);
}

static int
zydas_set_ssid(void *priv, u8 *buf, int len)
{
	struct zydas_driver_data *drv = priv;
	struct iwreq iwr;

	memset(&iwr, 0, sizeof(iwr));
	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
	iwr.u.essid.flags = 1; /* SSID active */
	iwr.u.essid.pointer = (caddr_t) buf;
	iwr.u.essid.length = len + 1;

	if (ioctl(drv->ioctl_sock, SIOCSIWESSID, &iwr) < 0) {
		perror("ioctl[SIOCSIWESSID]");
		printf("len=%d\n", len);
		return -1;
	}
	return 0;
}

static int
zydas_get_ssid(void *priv, u8 *buf, int len)
{
	struct zydas_driver_data *drv = priv;
	struct iwreq iwr;
	int ret = 0;

	memset(&iwr, 0, sizeof(iwr));
	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
	iwr.u.essid.pointer = (caddr_t) buf;
	iwr.u.essid.length = len;

	if (ioctl(drv->ioctl_sock, SIOCGIWESSID, &iwr) < 0) {
		perror("ioctl[SIOCGIWESSID]");
		ret = -1;
	} else
		ret = iwr.u.essid.length;

	return ret;
}

static int
zydas_set_countermeasures(void *priv, int enabled)
{
    struct zydas_driver_data *drv = priv;
    wpa_printf(MSG_ERROR, "%s: enabled=%d", __FUNCTION__, enabled);
    return zydas_set_param(drv, ZD_PARAM_COUNTERMEASURES, enabled, 1);

}

static const struct driver_ops zydas_driver_ops = {
	.name			= "zydas",
	.init			= zydas_init,
	.deinit			= zydas_deinit,
	.set_ieee8021x		= zydas_set_ieee8021x,
	.set_privacy		= zydas_set_privacy,
	.set_encryption		= zydas_set_key,
	.get_seqnum		= zydas_get_seqnum,
	.flush			= zydas_flush,
	.set_generic_elem	= zydas_set_opt_ie,
	.wireless_event_init	= zydas_wireless_event_init,
	.wireless_event_deinit	= zydas_wireless_event_deinit,
	.set_sta_authorized	= zydas_set_sta_authorized,
	.read_sta_data		= zydas_read_sta_driver_data,
	.send_eapol		= zydas_send_eapol,
	.sta_disassoc		= zydas_sta_disassoc,
	.sta_deauth		= zydas_sta_deauth,
	.set_ssid		= zydas_set_ssid,
	.get_ssid		= zydas_get_ssid,
        .set_countermeasures    = zydas_set_countermeasures,
#ifdef SIMPLE_CONFIG
        .set_wsc_beacon_ie	= zydas_set_wsc_beacon_ie,
        .set_wsc_probe_resp_ie	= zydas_set_wsc_probe_resp_ie,
#endif
};

void zydas_driver_register(void)
{
	driver_register(zydas_driver_ops.name, &zydas_driver_ops);
}

